@@ -1,6 +1,6 @@
 #######################################################################
 # Moving Generator -  Infrastructure for Moving Streams
-# Copyright (C) 2013 Michael Hahsler, Matthew Bolanos, John Forrest 
+# Copyright (C) 2013 Michael Hahsler, Matthew Bolanos, John Forrest
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,22 +16,22 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-### creator    
+### creator
 DSD_MG<- function(dimension = 2, ..., labels=NULL, description=NULL) {
-  
+
   if(is.null(description)) description <- "Moving Data Generator"
-  
+
   x <- structure(list(description = description,
     RObj = dsd_MG_refClass$new(d = dimension)),
     class = c("DSD_MG", "DSD_R", "DSD_data.frame", "DSD"))
-  
+
   l <- list(...)
   if(length(l) > 0) {
     for(i in 1:length(l)) {
       add_cluster(x, l[[i]], labels[i])
     }
-  } 
-  
+  }
+
   x
 }
 
@@ -39,7 +39,7 @@ add_cluster <- function(x, c, label=NULL) UseMethod("add_cluster")
 get_clusters <- function(x) UseMethod("get_clusters")
 remove_cluster <- function(x, i) UseMethod("remove_cluster")
 
-dsd_MG_refClass <- setRefClass("dsd_MG", 
+dsd_MG_refClass <- setRefClass("dsd_MG",
   fields = list(
     t = "numeric",
     dimension = "numeric",
@@ -54,7 +54,7 @@ dsd_MG_refClass <- setRefClass("dsd_MG",
       labels <<- integer(0)
       .self
     }
-    
+
   ),
 )
 
@@ -62,26 +62,26 @@ dsd_MG_refClass$methods(
   add_cluster = function(c, label=NULL) {
     if(c$RObj$dimension != dimension) stop("Cluster dimensions do not match!")
     clusters <<- append(clusters, list(c))
-    
+
     if(is.null(label)) {
       if(length(labels) == 0) label <- 1L
       else label <- max(labels, na.rm=TRUE) + 1L
       } else label <- as.integer(label)
     labels <<- append(labels, label)
     },
-  
-  get_points = function(n, cluster = FALSE) {
+
+  get_points = function(n, cluster = FALSE, outlier = FALSE) {
     if(length(clusters)==0) stop("DSD_MG does not contain any clusters!")
 
     if(cluster) a <- integer(n)
     data <- matrix(NA_real_, nrow=n, ncol=dimension)
-    
+
     j <- 0L
     while(j < n) {
-      
-      density <- unlist(sapply(clusters, 
+
+      density <- unlist(sapply(clusters,
         function(x) x$RObj$get_attributes(t, "density")))
-      
+
       density[is.na(density)] <- 0
       if(all(density==0)) stop("No MGC is producing points for this time point.")
 
@@ -89,54 +89,54 @@ dsd_MG_refClass$methods(
       pointsLeftInSecond <- pointsPerSecond - (t - floor(t))*pointsPerSecond
       if((j + pointsLeftInSecond) <= n) k <- pointsLeftInSecond
       else k <- n-j
-      
+
       ### got to next timestep...
       if(pointsLeftInSecond<1) {
         t <<- ceiling(t)
         next
       }
-      
+
       k <- floor(k)
-      
+
       if(k>=1) {
-        clusterOrder <- sample(x=1:length(clusters), size=k, replace=TRUE, 
+        clusterOrder <- sample(x=1:length(clusters), size=k, replace=TRUE,
           prob=density/sum(density))
-        
+
         data[(j+1):(j+k),] <- t(sapply(clusterOrder, FUN = function(i) {
           clusters[[i]]$RObj$get_points(t)
         }))
-        
+
         if(cluster) {
           a[(j+1):(j+k)] <- labels[clusterOrder]
         }
       }
-      
+
       t <<- t + k/pointsPerSecond
       j <- j+k
     }
-    
+
     data <- data.frame(data)
-    
+
     if(cluster) attr(data,"cluster") <- a
-    
+
     data
   }
 )
 
 
 
-get_points.DSD_MG <- function(x, n=1, 
-  outofpoints=c("stop", "warn", "ignore"), 
-  cluster = FALSE, class = FALSE, ...) {
+get_points.DSD_MG <- function(x, n=1,
+  outofpoints=c("stop", "warn", "ignore"),
+  cluster = FALSE, class = FALSE, outlier = FALSE, ...) {
   .nodots(...)
 
   d <- x$RObj$get_points(n, cluster=TRUE)
 
   a <- attr(d, "cluster")
   if(!cluster) attr(d, "cluster") <- NULL
-  
+
   if(class) d <- cbind(d, class = a)
-  
+
   d
 
 }
@@ -155,7 +155,7 @@ print.DSD_MG <- function(x, ...) {
   #NextMethod()
   cat(.line_break(paste(x$description)))
   cat("Class:", paste(class(x), collapse=", "), "\n")
-  cat(paste('With', length(na.omit(unique(x$RObj$labels))), 'clusters', 'in', 
+  cat(paste('With', length(na.omit(unique(x$RObj$labels))), 'clusters', 'in',
     x$RObj$dimension, 'dimensions. Time is', round(x$RObj$t, 3), '\n'))
 }
 
@@ -165,5 +165,5 @@ get_clusters.DSD_MG <- function(x) {
 
 remove_cluster.DSD_MG  <- function(x,i) {
   x$RObj$clusters[[i]] <- NULL
-  x$RObj$labels <- x$RObj$labels[-i] 
+  x$RObj$labels <- x$RObj$labels[-i]
 }
