@@ -18,7 +18,7 @@
 
 ### creator    
 MGC_Static <- function(density = 1, center, parameter, 
-  shape = NULL, cluster = NA) {
+  shape = NULL) {
     if(is.null(shape)) shape <- MGC_Shape_Gaussian
   
     x <- MGC_Function( 
@@ -34,3 +34,5 @@ MGC_Static <- function(density = 1, center, parameter,
   x
 }
 
+
+
